#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Fileliste erstellen für  wellenlängenkalibrierte 1d_Spektren im fits-Format
einer Zeitreihe. Plotten der Spektren. Beschneiden aller Spektren auf einen
wählbaren Wellenlängenbereich und abspeichern aller beschnittenen als fit oder
als ascii-Datei (tab und csv) mit Wellenlängenbereich im Dateinamen.


@author: lothar
20.1.2022
"""

import numpy as np
from astropy.io import fits, ascii
import matplotlib.pyplot as plt
import glob


# Fileliste erstellen. Spektren in einem (Unter)Ordner.
files = input('Pfad und Name der Spektren (nutze wildcards) : ')
filelist = glob.glob(files)

# Alphabetisches Sortieren. Bei richtiger Namensgebung ergibt das eine
# zeitliche Ordnung
filelist.sort()

# Ausdruck der Liste
print('\nSpektrenliste: \n')
print(filelist)
print('Anzahl der Spektren: ', len(filelist), '\n')


fig = plt.figure(figsize=(14, 20))
# Einlesen von flux und header:
for i in range(len(filelist)):
    flux, header = fits.getdata(filelist[i], header=True)
    # Grafik mit allen Spektrenauschnitten:
    wave = np.zeros(header['NAXIS1'], dtype=float)
    for k in range(len(wave)):
        wave[k] = header['CRVAL1'] + (k-header['CRPIX1']+1)*header['CDELT1']
    plt.plot(wave, flux, linewidth=1)
    if i == 0:
        if 'OBJECT' not in header:
            obj = input('Geben Sie den Namen des Beobachtungsobjekts ein: ')
        else:
            obj = header['OBJECT']

# Ploteigenschaften (Gitter, Beschriftungen etc.) anpassen:
plt.title('Zeitserie von ' + obj)
plt.grid(True)
plt.xlabel('Wellenlänge [Angström]')
# Legende kann angepasst werden, z.B. Schriftgröße (fontsize)
plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
           ncol=4, mode="expand", borderaxespad=0., fontsize=6)

plt.show(block=False)

# Grafiken speichern
# fig.savefig(header['OBJECT']+'_Zeitserie.pdf')
# fig.savefig(header['OBJECT']+'_Zeitserie.png')


# Generate the spectrum section, plot and save as fit

min = 0
max = 10000
for i in range(len(filelist)):
    sp = fits.open(filelist[i])
    if sp[0].header['CRVAL1'] > min:
        min = sp[0].header['CRVAL1']
    if sp[0].header['CRVAL1'] + sp[0].header['CDELT1']*sp[0].header['NAXIS1'] < max:
        max = sp[0].header['CRVAL1'] + \
            sp[0].header['CDELT1']*sp[0].header['NAXIS1']
    sp.close()
print('\nGemeinsamer Wellenlängenbereich: ', min, max)

print('\nSpecification of the wavelength range to be transferred ')
a = float(input('Begin: '))
b = float(input('End: '))

for i in range(len(filelist)):
    sp = fits.open(filelist[i])
    step = sp[0].header['CDELT1']
    refpix = sp[0].header['CRPIX1']
    wave_erstesPix = sp[0].header['CRVAL1'] - step*(refpix - 1)
    aindex = int((a - wave_erstesPix) / step)
    bindex = int((b - wave_erstesPix) / step)
    a = sp[0].header['CRVAL1'] + aindex * step
    b = sp[0].header['CRVAL1'] + bindex * step
    filename = filelist[i].rsplit(
        '.')[0]+'_'+str(int(a))+'_'+str(int(b))+'.fit'
    newflux = sp[0].data[aindex:bindex]
    sp[0].header['CRVAL1'] = a
    sp[0].header['NAXIS1'] = bindex - aindex
    sp[0].header['CRPIX1'] = 1
    fits.writeto(filename, newflux, sp[0].header,
                 overwrite=True, output_verify='silentfix')
    newwave = np.zeros(sp[0].header['NAXIS1'], dtype=float)
    for k in range(len(newwave)):
        newwave[k] = a + k * step
    # # filename = filelist[i].rsplit('.')[0]+'_'+str(int(a))+'_'+str(int(b))+'.csv'
    # ascii.write([newwave, newflux], filename, overwrite=True,
    #         names=['WAVE', 'FLUX'], format='csv')
    filename = filelist[i].rsplit(
        '.')[0]+'_'+str(int(a))+'_'+str(int(b))+'.dat'
    ascii.write([newwave, newflux], filename, overwrite=True,
                names=['WAVE', 'FLUX'], format='tab')
    sp.close()
